@@ -50,6 +50,7 @@ class Source(enum.Enum):
     WYSE     = enum.auto()
     XDG      = enum.auto()
     XTERM    = enum.auto()
+    UNDET    = enum.auto()
 
     @classmethod
     def from_name(cls, name):
@@ -80,7 +81,7 @@ class Source(enum.Enum):
         elif name.startswith('XTERM'):
             return cls.XTERM
         else:
-            return cls.ECMA48
+            return cls.UNDET
             #raise ValueError(f'Could not determine source for mode {name}')
 
 # Control Sequence
@@ -694,7 +695,7 @@ modes = [
     #
     # aka DECVSSM
     #
-    mode_WHAT('DECLRMM', 69, default=False),
+    mode_WHAT('DECLRMM', 69, default=False, flags=Flags.WRITABLE),
 
     # DECXRLM - transmit rate limit
     # If set, limits the transmit rate; if reset, the rate is
@@ -991,6 +992,15 @@ modes = [
 
     mode_WHAT('CONTOUR_BATCHED_RENDERING', 2026, default=False),
 
+    # "Unicode Core"
+    # https://github.com/contour-terminal/terminal-unicode-core
+    mode_WHAT('CONTOUR_UNICODE_CORE', 2027, default=False),
+
+    mode_WHAT('CONTOUR_TEXT_REFLOW', 2028, default=False),
+    mode_WHAT('CONTOUR_MOUSE_PASSIVE_TRACKING', 2029, default=False),
+    mode_WHAT('CONTOUR_GRID_CELL_SELECTION', 2030, default=False),
+    mode_WHAT('CONTOUR_COLOUR_PALETTE_REPORTS', 2031, default=False),
+
     # DRCSTerm:
 
     # DRCSMM_V1
@@ -1187,6 +1197,9 @@ modes = [
     mode_WHAT('XTERM_MOUSE_EXT_SGR', 1006, default=False, flags=Flags.WRITABLE),
     mode_WHAT('XTERM_ALTBUF_SCROLL', 1007, default=True, flags=Flags.WRITABLE),
 
+    mode_WHAT('XTERM_FAST_SCROLL', 1014, default=False),
+    mode_WHAT('XTERM_MOUSE_EXT_SGR_PIXEL', 1016, default=False),
+
     mode_WHAT('XTERM_8BIT_META', 1034, default=False),
     mode_WHAT('XTERM_NUMLOCK', 1035, default=False),
 
@@ -1238,6 +1251,11 @@ modes = [
     mode_WHAT('XTERM_READLINE_PASTE_QUOTE', 2005, default=False),
     mode_WHAT('XTERM_READLINE_PASTE_LITERAL_NL', 2006, default=False),
 
+    # In-band resize notifications,
+    #
+    # References: https://gist.github.com/rockorager/e695fb2924d36b2bcf1fff4a3704bd83
+    mode_WHAT('UNDET_IRN', 2048, default=False),
+
     # ************************************************************************
 
     # Modes for SM_HP/RM_HP (CSI > n [h|l])
