@@ -776,7 +776,7 @@ sequences = [
             comment='pcterm'),
     seq_CSI('XTERM_RPM', 'r', pintro=(ParameterIntro.WHAT,),
             comment='xterm restore DEC private mode'),
-    seq_CSI('DECSLRM', 's', flags=Flags.NOP,
+    seq_CSI('DECSLRM', 's',
             comment='set left and right margins'),
     seq_CSI('SCOSC', 's',
             comment='SCO save cursor'),
@@ -922,14 +922,17 @@ sequences = [
             comment='restore terminal state'),
     seq_DCS('XTERM_STCAP', 'p', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
             comment='xterm set termcap/terminfo'),
-    seq_DCS('DECSIXEL', 'q', flags=Flags.NOP,
+    seq_DCS('DECSIXEL', 'q', flags=Flags.UNRIPE | Flags.HANDLER_RV,
             comment='SIXEL graphics'),
     seq_DCS('DECRQSS', 'q', intermediates=(Intermediate.CASH,),
             comment='request selection or setting'),
-    seq_DCS('XTERM_RQTCAP', 'q', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
+    seq_DCS('XTERM_RQTCAP', 'q', intermediates=(Intermediate.PLUS,),
             comment='xterm request termcap/terminfo'),
     seq_DCS('DECLBAN', 'r', flags=Flags.NOP,
             comment='load banner message'),
+    seq_DCS('XTERM_TCAPR', 'r', intermediates=(Intermediate.PLUS,), flags=Flags.NOP,
+            direction=Direction.TTH,
+            comment='xterm termcap report'),
     seq_DCS('DECTSR', 's', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             direction = Direction.TTH,
             comment='terminal state report'),
