@@ -21,16 +21,20 @@
 #error "Only <vte/vte.h> can be included directly."
 #endif
 
+#include <stdint.h>
+
 #include <glib.h>
 #include <gio/gio.h>
 #include <cairo.h>
 #include <pango/pango.h>
 #include <gtk/gtk.h>
 
-#include "vteenums.h"
 #include "vtemacros.h"
+#include "vteenums.h"
+#include "vteproperties.h"
 #include "vtepty.h"
 #include "vteregex.h"
+#include "vteuuid.h"
 
 G_BEGIN_DECLS
 
@@ -119,16 +123,24 @@ struct _VteTerminalClass {
         gpointer _extra_padding[3];
 #endif /* _VTE_GTK == 3 */
 
+        /*< protected >*/
         void (*setup_context_menu)(VteTerminal* terminal,
                                    VteEventContext const* context);
 
+        gboolean (* termprops_changed)(VteTerminal* terminal,
+                                       int const* props,
+                                       int n_props);
+
+        void (* termprop_changed)(VteTerminal* terminal,
+                                  char const* prop);
+
         /* Add new vfuncs just above, and subtract from the padding below. */
 
         /* Padding for future expansion. */
 #if _VTE_GTK == 3
-        gpointer _padding[12];
+        gpointer _padding[10];
 #elif _VTE_GTK == 4
-        gpointer _padding[15];
+        gpointer _padding[13];
 #endif /* _VTE_GTK */
 
 // FIXMEgtk4 use class private data instead
@@ -388,6 +400,13 @@ _VTE_PUBLIC
 void vte_terminal_set_delete_binding(VteTerminal *terminal,
 				     VteEraseBinding binding) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
 
+/* Accessibility */
+_VTE_PUBLIC
+void vte_terminal_set_enable_a11y(VteTerminal *terminal,
+                                  gboolean enable_a11y) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+_VTE_PUBLIC
+gboolean vte_terminal_get_enable_a11y(VteTerminal *terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
 /* BiDi and shaping */
 _VTE_PUBLIC
 void vte_terminal_set_enable_bidi(VteTerminal *terminal,
@@ -556,12 +575,6 @@ _VTE_PUBLIC
 glong vte_terminal_get_row_count(VteTerminal *terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
 _VTE_PUBLIC
 glong vte_terminal_get_column_count(VteTerminal *terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
-_VTE_PUBLIC
-const char *vte_terminal_get_window_title(VteTerminal *terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
-_VTE_PUBLIC
-const char *vte_terminal_get_current_directory_uri(VteTerminal *terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
-_VTE_PUBLIC
-const char *vte_terminal_get_current_file_uri(VteTerminal *terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
 
 /* misc */
 _VTE_PUBLIC
@@ -579,6 +592,9 @@ _VTE_PUBLIC
 void vte_terminal_get_color_background_for_draw(VteTerminal* terminal,
                                                 GdkRGBA* color) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
 
+_VTE_PUBLIC
+void vte_terminal_set_suppress_legacy_signals(VteTerminal* terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
 /* Writing contents out */
 _VTE_PUBLIC
 gboolean vte_terminal_write_contents_sync (VteTerminal *terminal,
@@ -625,6 +641,13 @@ void vte_terminal_set_yfill(VteTerminal* terminal,
 _VTE_PUBLIC
 gboolean vte_terminal_get_yfill(VteTerminal* terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
 
+_VTE_PUBLIC
+void vte_terminal_set_enable_legacy_osc777(VteTerminal* terminal,
+                                           gboolean enable) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_enable_legacy_osc777(VteTerminal* terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
 _VTE_PUBLIC
 void vte_terminal_set_context_menu_model(VteTerminal* terminal,
                                          GMenuModel* model) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
@@ -656,6 +679,186 @@ gboolean vte_event_context_get_coordinates(VteEventContext const* context,
 
 #endif /* VTE_GTK */
 
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_bool(VteTerminal* terminal,
+                                        char const* prop,
+                                        gboolean* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_bool_by_id(VteTerminal* terminal,
+                                              int prop,
+                                              gboolean* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_int(VteTerminal* terminal,
+                                       char const* prop,
+                                       int64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_int_by_id(VteTerminal* terminal,
+                                             int prop,
+                                             int64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_uint(VteTerminal* terminal,
+                                        char const* prop,
+                                        uint64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_uint_by_id(VteTerminal* terminal,
+                                              int prop,
+                                              uint64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_double(VteTerminal* terminal,
+                                          char const* prop,
+                                          double* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_double_by_id(VteTerminal* terminal,
+                                                int prop,
+                                                double* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_rgba(VteTerminal* terminal,
+                                        char const* prop,
+                                        GdkRGBA* color) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_rgba_by_id(VteTerminal* terminal,
+                                              int prop,
+                                              GdkRGBA* color) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+char const* vte_terminal_get_termprop_string(VteTerminal* terminal,
+                                             char const* prop,
+                                             size_t* size) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+char const* vte_terminal_get_termprop_string_by_id(VteTerminal* terminal,
+                                                   int prop,
+                                                   size_t* size) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+char* vte_terminal_dup_termprop_string(VteTerminal* terminal,
+                                       char const* prop,
+                                       size_t* size) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+char* vte_terminal_dup_termprop_string_by_id(VteTerminal* terminal,
+                                             int prop,
+                                             size_t* size) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+uint8_t const* vte_terminal_get_termprop_data(VteTerminal* terminal,
+                                              char const* prop,
+                                              size_t* size) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2, 3);
+
+_VTE_PUBLIC
+uint8_t const* vte_terminal_get_termprop_data_by_id(VteTerminal* terminal,
+                                                    int prop,
+                                                    size_t* size) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 3);
+
+_VTE_PUBLIC
+GBytes* vte_terminal_ref_termprop_data_bytes(VteTerminal* terminal,
+                                             char const* prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+GBytes* vte_terminal_ref_termprop_data_bytes_by_id(VteTerminal* terminal,
+                                                   int prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+VteUuid* vte_terminal_dup_termprop_uuid(VteTerminal* terminal,
+                                        char const* prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+VteUuid* vte_terminal_dup_termprop_uuid_by_id(VteTerminal* terminal,
+                                              int prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+GUri* vte_terminal_ref_termprop_uri(VteTerminal* terminal,
+                                    char const* prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+GUri* vte_terminal_ref_termprop_uri_by_id(VteTerminal* terminal,
+                                          int prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+cairo_surface_t* vte_terminal_ref_termprop_image_surface(VteTerminal* terminal,
+                                                         char const* prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+cairo_surface_t* vte_terminal_ref_termprop_image_surface_by_id(VteTerminal* terminal,
+                                                               int prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+#if _VTE_GTK == 3
+
+_VTE_PUBLIC
+GdkPixbuf* vte_terminal_ref_termprop_image_pixbuf(VteTerminal* terminal,
+                                                  char const* prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+GdkPixbuf* vte_terminal_ref_termprop_image_pixbuf_by_id(VteTerminal* terminal,
+                                                        int prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+#elif _VTE_GTK == 4
+
+_VTE_PUBLIC
+GdkTexture* vte_terminal_ref_termprop_image_texture(VteTerminal* terminal,
+                                                    char const* prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+GdkTexture* vte_terminal_ref_termprop_image_texture_by_id(VteTerminal* terminal,
+                                                          int prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+#endif /* _VTE_GTK */
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_value(VteTerminal* terminal,
+                                         char const* prop,
+                                         GValue* gvalue) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2, 3);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_value_by_id(VteTerminal* terminal,
+                                               int prop,
+                                               GValue* gvalue) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 3);
+
+_VTE_PUBLIC
+GVariant* vte_terminal_ref_termprop_variant(VteTerminal* terminal,
+                                            char const* prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+GVariant* vte_terminal_ref_termprop_variant_by_id(VteTerminal* terminal,
+                                                  int prop) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_enum(VteTerminal* terminal,
+                                        char const* prop,
+                                        GType gtype,
+                                        int64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_enum_by_id(VteTerminal* terminal,
+                                              int prop,
+                                              GType gtype,
+                                              int64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_flags(VteTerminal* terminal,
+                                         char const* prop,
+                                         GType gtype,
+                                         gboolean ignore_unknown_flags,
+                                         uint64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1, 2);
+
+_VTE_PUBLIC
+gboolean vte_terminal_get_termprop_flags_by_id(VteTerminal* terminal,
+                                               int prop,
+                                               GType gtype,
+                                               gboolean ignore_unknown_flags,
+                                               uint64_t* valuep) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
+VteProperties const* vte_terminal_get_termprops(VteTerminal* terminal) _VTE_CXX_NOEXCEPT _VTE_GNUC_NONNULL(1);
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(VteTerminal, g_object_unref)
 
 G_END_DECLS
